--
-- VeG-S - VehicleGroups Switcher - (previously known as "FastSwitcher")
--
-- @author  Decker_MMIV - fs-uk.com, forum.farming-simulator.com, modhoster.com
-- @date    2012-11-17
--
-- @history
--  2012-November
--      v0.9      - Took FastSwitcher v1.3 from FS2011, and changed alot.
--      v0.91     - Added "switch within group", suggested by Knut & KaosKnite.
--      v0.92     - Minor tweaks. Could also be used in FS2011, if modified slightly.
--


VehicleGroupsSwitcher = {};
VehicleGroupsSwitcher.hudPosSize = {x1=0.15, x2=0.55, x=0.37, y=0.8, w=1, h=0.8}; -- X,Y,Width,Height        -- TODO: Make position customizable from within the game.
VehicleGroupsSwitcher.bigFontSize   = 0.023;
VehicleGroupsSwitcher.smallFontSize = 0.020;
VehicleGroupsSwitcher.fontColor         = {1.0, 1.0, 1.0, 1.0}; -- white
VehicleGroupsSwitcher.fontSelectedColor = {1.0, 1.0, 0.3, 1.0}; -- yellowish
VehicleGroupsSwitcher.fontShadeColor    = {0.0, 0.0, 0.0, 1.0}; -- black


-- Register as event listener
addModEventListener(VehicleGroupsSwitcher);

--
--
--

function VehicleGroupsSwitcher:loadSteerable(xmlFile)
  -- Try to find the name of the vehicle.
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle.name.".. g_languageShort); end
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle.name"); end
  if self.name == nil then self.name = getXMLString(xmlFile, "vehicle#type"); end
  if self.name == nil then
      self.name = "Steerable" -- Oh well, lets just call it this then!
  end
end
Steerable.load  = Utils.appendedFunction(Steerable.load, VehicleGroupsSwitcher.loadSteerable);

--
--
--

-- FS2013
-- Support-function, that I would like to see be added to InputBinding class.
-- Maybe it is, I just do not know what its called.
function getKeyIdOfModifier(binding)
    if InputBinding.actions[binding] == nil then
        return nil;  -- Unknown input-binding.
    end;
    if table.getn(InputBinding.actions[binding].keys1) <= 1 then
        return nil; -- Input-binding has only one or zero keys. (Well, in the keys1 - I'm not checking keys2)
    end;
    -- Check if first key in key-sequence is a modifier key (LSHIFT/RSHIFT/LCTRL/RCTRL/LALT/RALT)
    if Input.keyIdIsModifier[ InputBinding.actions[binding].keys1[1] ] then
        return InputBinding.actions[binding].keys1[1]; -- Return the keyId of the modifier key
    end;
    return nil;
end
--]]

--[[ FS2011
--http://stackoverflow.com/questions/656199/search-for-an-item-in-a-lua-list
function Set(list)
    local set = {};
    for _,l in ipairs(list) do
        set[l]=true;
    end;
    return set;
end;

function getKeyIdOfModifier(binding)
    local allowedModifiers = Set({
        Input.KEY_lshift,
        Input.KEY_rshift,
        Input.KEY_shift,
        Input.KEY_lctrl, 
        Input.KEY_rctrl, 
        Input.KEY_lalt,  
        Input.KEY_ralt  
    });
    for _,k in pairs(InputBinding.digitalActions[binding].key1Modifiers) do
        if allowedModifiers[k] then
            return k;
        end;
    end;
    return nil;
end;
--]]

--
--
--

function VehicleGroupsSwitcher:loadMap(name)
    if self.initialized then
        return;
    end;
    --
    self.keyModifier = getKeyIdOfModifier(InputBinding.VEGS_TOGGLE_EDIT);
    
    if not (    self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_01)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_02)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_03)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_04)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_05)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_06)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_07)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_08)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_09)
            and self.keyModifier == getKeyIdOfModifier(InputBinding.VEGS_GRP_10))
    then
        print("ERROR: One-or-more inputbindings for VehicleGroupsSwitcher do not use the same modifier-key (SHIFT/CTRL/ALT)!");
        return;
    end;

-- FS2013    
    self.hudBackground = createImageOverlay("dataS2/menu/black.png");
--]]    
    --
    self.initialized = true;
end;

function VehicleGroupsSwitcher:deleteMap()
    self.initialized = nil;
    --
    delete(self.hudBackground);
    self.hudBackground = nil;
end;

function VehicleGroupsSwitcher:mouseEvent(posX, posY, isDown, isUp, button)
end;

function VehicleGroupsSwitcher:keyEvent(unicode, sym, modifier, isDown)
end

function VehicleGroupsSwitcher:update(dt)
    if not self.initialized then
        return;
    end;
    --
    -- Only the server-host has the ability to move vehicles to different groups.
    if g_server ~= nil then
        if g_currentMission.showHelpText then
            if self.isModifying or ((self.keyModifier == nil) or (Input.isKeyPressed(self.keyModifier))) then
                g_currentMission:addHelpButtonText(g_i18n:getText("VEGS_TOGGLE_EDIT"), InputBinding.VEGS_TOGGLE_EDIT);
                if self.isModifying and not g_currentMission.player.isEntered then
                    g_currentMission:addHelpButtonText(g_i18n:getText("editUp"),   InputBinding.MENU_UP);
                    g_currentMission:addHelpButtonText(g_i18n:getText("editDown"), InputBinding.MENU_DOWN);
                end;
            end;
        end;
        --
        if InputBinding.hasEvent(InputBinding.VEGS_TOGGLE_EDIT) or self.isModifying then 
            if self.isModifying then
                local vehGroupOffset = nil;
                if     InputBinding.hasEvent(InputBinding.MENU_UP)   then vehGroupOffset = -1;
                elseif InputBinding.hasEvent(InputBinding.MENU_DOWN) then vehGroupOffset =  1;
                end
                --
                if vehGroupOffset ~= nil then
                    for _,vehObj in pairs(g_currentMission.steerables) do
                        if vehObj.isEntered then
                            vehObj.vegsGroup = ((vehObj.vegsGroup or 0) + vehGroupOffset) % 11;
                            self.dirtyTimeout = g_currentMission.time + 2000; -- broadcast update, after 2 seconds have passed from now
                            break;
                        end;
                    end;
                end;
                --
                if (g_gui.currentGuiName ~= "" and g_gui.currentGuiName ~= nil) then
                    -- If player activates some GUI screen, stop VEGS from rendering
                    self.isModifying = false;
                else
                    self.isModifying = not InputBinding.hasEvent(InputBinding.VEGS_TOGGLE_EDIT);
                end;
            else
                self.isModifying = true;
            end;
        end;
        --
-- FS2013        
        if g_currentMission.isSychronizingWithPlayers then
            -- Do not know if this really works, but else just have the server-host make a small edit after a client has joined.        
--if self.dirtyTimeout == nil then
--    print(tostring(g_currentMission.time).. "ms - g_currentMission.isSychronizingWithPlayers = true!");
--end;
            self.dirtyTimeout = g_currentMission.time + 5000; -- broadcast, due to newly connected player(s)
        end;
--]]        
        if self.dirtyTimeout ~= nil and self.dirtyTimeout < g_currentMission.time then
            self.dirtyTimeout = nil;
            VehicleGroupsSwitcherEvent.sendEvent();
        end;
    end;
    --
    -- This construct is used, so we do not activate other actions that might have been assigned the normal-keys (i.e. 1,2,3...9,0)
    local vegsSwitchTo = nil;
    if InputBinding.hasEvent(InputBinding.VEGS_GRP_TAB) then
        -- Switch within the same group (if possible)
        for _,vehObj in pairs(g_currentMission.steerables) do
            if vehObj.isEntered then
                vegsSwitchTo = vehObj.vegsGroup;
                break;
            end;
        end;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_01) then vegsSwitchTo =  1;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_02) then vegsSwitchTo =  2; 
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_03) then vegsSwitchTo =  3;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_04) then vegsSwitchTo =  4;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_05) then vegsSwitchTo =  5;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_06) then vegsSwitchTo =  6;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_07) then vegsSwitchTo =  7;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_08) then vegsSwitchTo =  8;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_09) then vegsSwitchTo =  9;
    elseif InputBinding.hasEvent(InputBinding.VEGS_GRP_10) then vegsSwitchTo = 10;
    end;
    --
    if vegsSwitchTo ~= nil then
        local vehStartIdx = 0;
        for idx,vehObj in pairs(g_currentMission.steerables) do
            if vehObj.isEntered then
                if vegsSwitchTo == vehObj.vegsGroup then -- Special case, if switching within the same slot-group.
                    vehStartIdx = idx;
                end;
                break;
            end;
        end;
        local numVeh = table.getn(g_currentMission.steerables);
        local foundVehObj = nil;
        for idx = vehStartIdx, numVeh+vehStartIdx do
            local vehObj = g_currentMission.steerables[(idx % numVeh)+1];
            if vehObj.vegsGroup ~= nil and vehObj.vegsGroup == vegsSwitchTo then
                if not vehObj.isEntered and not vehObj.isControlled then
                    foundVehObj = vehObj;
                    break;
                end;
            end;
        end;
        if foundVehObj then
            g_client:getServerConnection():sendEvent(VehicleEnterRequestEvent:new(foundVehObj, g_settingsNickname));
        end;
    end;
end;

function renderTextWithShade(x,y, textSize, forecolor, text)
    setTextColor(unpack(VehicleGroupsSwitcher.fontShadeColor));
    renderText(x + (textSize/20), y - (textSize/10), textSize, text);
    setTextColor(unpack(forecolor));
    renderText(x, y, textSize, text);
end;

function VehicleGroupsSwitcher:draw()
    if not self.initialized then
        if InputBinding.isPressed(InputBinding.VEGS_TOGGLE_EDIT) then
            setTextAlignment(RenderText.ALIGN_CENTER);
            renderTextWithShade(0.5, 0.7, VehicleGroupsSwitcher.bigFontSize, VehicleGroupsSwitcher.fontColor, g_i18n:getText("ControlsError"));
        end;
        return;
    end;
    --
    if self.isModifying or ((self.keyModifier ~= nil) and (Input.isKeyPressed(self.keyModifier))) then
        local slots = {}
        local unassigned = {}
        for idx=1,10 do
            slots[idx] = {}
        end;
        for idx,vehObj in pairs(g_currentMission.steerables) do
            if vehObj.vegsGroup ~= nil and vehObj.vegsGroup >= 1 and vehObj.vegsGroup <= 10 then
                table.insert(slots[vehObj.vegsGroup], vehObj);
            else
                table.insert(unassigned, vehObj);
            end;
        end;
        --
        local xPos = 0.5;
        local yPos = VehicleGroupsSwitcher.hudPosSize.y;
        local yPosLowest = 0.5;
        --
-- FS2013        
        if self.hudBackground ~= nil then
            setOverlayColor(self.hudBackground, 1,1,1, 0.4);
            renderOverlay(self.hudBackground, 0.0,0.33, 1.0,0.50);
        end;
--]]
        --
        setTextBold(true);
        setTextAlignment(RenderText.ALIGN_CENTER);
        renderTextWithShade(xPos, yPos, VehicleGroupsSwitcher.bigFontSize, VehicleGroupsSwitcher.fontColor, g_i18n:getText("VEGS"));
        setTextAlignment(RenderText.ALIGN_LEFT);
        --
        for idx=1,10 do
            if idx == 1 then
                xPos = VehicleGroupsSwitcher.hudPosSize.x1;
                yPos = VehicleGroupsSwitcher.hudPosSize.y - VehicleGroupsSwitcher.smallFontSize;
            elseif idx == 6 then
                yPosLowest = math.min(yPos, yPosLowest);
                xPos = VehicleGroupsSwitcher.hudPosSize.x2;
                yPos = VehicleGroupsSwitcher.hudPosSize.y - VehicleGroupsSwitcher.smallFontSize;
            end;
            --
            setTextBold(true);
            yPos = yPos - VehicleGroupsSwitcher.bigFontSize;
            renderTextWithShade(xPos, yPos, VehicleGroupsSwitcher.bigFontSize, VehicleGroupsSwitcher.fontColor, string.format(g_i18n:getText("group"), idx));
            --
            setTextBold(false);
            for _,vehObj in pairs(slots[idx]) do
                local color = (vehObj.isEntered and VehicleGroupsSwitcher.fontSelectedColor or VehicleGroupsSwitcher.fontColor);
                yPos = yPos - VehicleGroupsSwitcher.smallFontSize;
                renderTextWithShade(xPos + VehicleGroupsSwitcher.smallFontSize, yPos, VehicleGroupsSwitcher.smallFontSize, color, tostring(vehObj.name));
                --
                if vehObj.isControlled
                or vehObj.isHired  -- Hired helper
                or (vehObj.drive ~= nil and vehObj.drive == true)  -- CoursePlay
                then
                    local txt;
                    if vehObj.isControlled then
                        txt = vehObj.controllerName;
                        if txt == nil then
                            txt = g_i18n:getText("player");
                        end;
                    elseif vehObj.isHired then
                        txt = g_i18n:getText("hired");
                    elseif (vehObj.drive ~= nil and vehObj.drive == true) then
                        txt = g_i18n:getText("courseplay");
                    else
                        txt = g_i18n:getText("unknown");
                    end;
                    setTextAlignment(RenderText.ALIGN_RIGHT);
                    renderTextWithShade(xPos, yPos, VehicleGroupsSwitcher.smallFontSize, color, txt);
                    setTextAlignment(RenderText.ALIGN_LEFT);
                end;
            end;
        end;
        --
        if self.isModifying then
            xPos = VehicleGroupsSwitcher.hudPosSize.x;
            yPos = math.min(yPos, yPosLowest);
            --
            setTextBold(true);
            yPos = yPos - VehicleGroupsSwitcher.bigFontSize;
            renderTextWithShade(xPos, yPos, VehicleGroupsSwitcher.bigFontSize, VehicleGroupsSwitcher.fontColor, g_i18n:getText("unassigned"));
            --
            setTextBold(false);
            for _,vehObj in pairs(unassigned) do
                local color = (vehObj.isEntered and VehicleGroupsSwitcher.fontSelectedColor or VehicleGroupsSwitcher.fontColor);
                yPos = yPos - VehicleGroupsSwitcher.smallFontSize;
                renderTextWithShade(xPos + VehicleGroupsSwitcher.smallFontSize, yPos, VehicleGroupsSwitcher.smallFontSize, color, tostring(vehObj.name));
                --
                if vehObj.isControlled
                or vehObj.isHired  -- Hired helper
                or (vehObj.drive ~= nil and vehObj.drive == true)  -- CoursePlay
                then
                    local txt;
                    if vehObj.isControlled then
                        txt = vehObj.controllerName;
                        if txt == nil then
                            txt = g_i18n:getText("player");
                        end;
                    elseif vehObj.isHired then
                        txt = g_i18n:getText("hired");
                    elseif (vehObj.drive ~= nil and vehObj.drive == true) then
                        txt = g_i18n:getText("courseplay");
                    else
                        txt = g_i18n:getText("unknown");
                    end;
                    setTextAlignment(RenderText.ALIGN_RIGHT);
                    renderTextWithShade(xPos, yPos, VehicleGroupsSwitcher.smallFontSize, color, txt);
                    setTextAlignment(RenderText.ALIGN_LEFT);
                end;
            end;
        end;
    end;
end;

--
function VehicleGroupsSwitcher.loadFromAttributesAndNodes(self, superFunc, xmlFile, key, resetVehicles)
	if not resetVehicles and g_server ~= nil then
		self.vegsGroup = getXMLInt(xmlFile, key .. string.format("#vegs_group"));
    end;
    --
    return superFunc(self, xmlFile, key, resetVehicles);
end;

function VehicleGroupsSwitcher.getSaveAttributesAndNodes(self, superFunc, nodeIdent)
    local attributes;
    local nodes;
    attributes, nodes = superFunc(self, nodeIdent);
    --
    if self.vegsGroup ~= nil and self.vegsGroup > 0 then
        attributes = attributes .. ' vegs_group="'..tostring(self.vegsGroup)..'"';
    end;
    --
	return attributes, nodes;
end;

Vehicle.loadFromAttributesAndNodes = Utils.overwrittenFunction(Vehicle.loadFromAttributesAndNodes, VehicleGroupsSwitcher.loadFromAttributesAndNodes);
Vehicle.getSaveAttributesAndNodes  = Utils.overwrittenFunction(Vehicle.getSaveAttributesAndNodes,  VehicleGroupsSwitcher.getSaveAttributesAndNodes);

---
---
---

VehicleGroupsSwitcherEvent = {};
VehicleGroupsSwitcherEvent_mt = Class(VehicleGroupsSwitcherEvent, Event);

InitEventClass(VehicleGroupsSwitcherEvent, "VehicleGroupsSwitcherEvent");

function VehicleGroupsSwitcherEvent:emptyNew()
    local self = Event:new(VehicleGroupsSwitcherEvent_mt);
    self.className="VehicleGroupsSwitcherEvent";
    return self;
end;

function VehicleGroupsSwitcherEvent:new()
    local self = VehicleGroupsSwitcherEvent:emptyNew()
    return self;
end;

function VehicleGroupsSwitcherEvent:writeStream(streamId, connection)
--print(tostring(g_currentMission.time).. "ms - VehicleGroupsSwitcherEvent:writeStream(streamId, connection)");
    local cnt = table.getn(g_currentMission.steerables);
    streamWriteInt8(streamId, cnt); -- If more than 127 steerables, then this will be a problem!
    for i=1,cnt do
        local vegsGroup = g_currentMission.steerables[i].vegsGroup or 0;
        streamWriteInt32(streamId, networkGetObjectId(g_currentMission.steerables[i]));
        streamWriteInt8(streamId, vegsGroup);
    end
end;

function VehicleGroupsSwitcherEvent:readStream(streamId, connection)
--print(tostring(g_currentMission.time).. "ms - VehicleGroupsSwitcherEvent:readStream(streamId, connection)");
    local cnt = streamReadInt8(streamId);
    for i=1,cnt do
        local vehObj = networkGetObject(streamReadInt32(streamId));
        local vegsGroup = streamReadInt8(streamId);
        if vehObj ~= nil then
            vehObj.vegsGroup = vegsGroup;
        end;
    end;
end;

function VehicleGroupsSwitcherEvent.sendEvent(noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(VehicleGroupsSwitcherEvent:new(), nil, nil, nil);
        --else
        --    g_client:getServerConnection():sendEvent(VehicleGroupsSwitcherEvent:new());
        end;
    end;
end;

--
print("Script loaded: VehicleGroupsSwitcher.lua (v0.92)");
